-----------------------
--- Module Creation ---
-----------------------
local core = LibStub("AceAddon-3.0"):GetAddon("tomQuest2")
local module = core:NewModule("questsGivers", "AceEvent-3.0")

-----------------------
--- Library loading ---
-----------------------
local LQG = LibStub("AceLocale-3.0"):GetLocale("tomQuest2-QuestsGivers")
local LDQ = LibStub("AceLocale-3.0"):GetLocale("tomQuest2-DailyQuests")
local LFQ = LibStub("AceLocale-3.0"):GetLocale("tomQuest2-ForbidenQuests")

--------------------------------------------
--- Copy global functions to local scope ---
--------------------------------------------
local select, pairs, format = select, pairs, string.format
local GetGossipAvailableQuests, GetGossipActiveQuests = GetGossipAvailableQuests, GetGossipActiveQuests
local GetNumActiveQuests, GetNumAvailableQuests = GetNumActiveQuests, GetNumAvailableQuests
local GetActiveTitle, GetActiveLevel = GetActiveTitle, GetActiveLevel
local GetAvailableTitle, GetAvailableLevel = GetAvailableTitle, GetAvailableLevel
local AcceptQuest, IsShiftKeyDown, GetTitleText, GetNumQuestChoices, GetQuestReward, IsQuestCompletable, CompleteQuest, ConfirmAcceptQuest = AcceptQuest, IsShiftKeyDown, GetTitleText, GetNumQuestChoices, GetQuestReward, IsQuestCompletable, CompleteQuest, ConfirmAcceptQuest

-----------------
--- variables ---
-----------------
local prdb, options

module.modName = LQG["Quests givers"]

module.dailies = {}
module.automaticAcceptChoices = {[1] = LQG["All quests"], [2] = LQG["Only daily quests"], [3] = LQG["All but daily quests"], [4] = LQG["Never"]}

-----------------------
--- Default profile ---
-----------------------
local defaults = {
	profile = {
		addQuestLevel = true,
		fadeNotCompletedQuests = true,
		automaticAccept = 4,
		automaticAcceptEscort = false,
		automaticTurnIn = false,
	}
}

---------------
--- Options ---
---------------
local optGetter, optSetter
do
	function optGetter(info)
		return prdb[info[#info]]
	end
	function optSetter(info, value)
		prdb[info[#info]] = value
	end
end

local function getOptions()
	if not options then
		options = {
			type = "group",
			childGroups = "tab",
			name = module.modName,
			get = optGetter,
			set = optSetter,
			args = {
				addQuestLevel = {
					name = LQG["Add quest level to the quest name"],
					type = "toggle",
					width = "full",
					order = 1,
				},
				fadeNotCompletedQuests = {
					name = LQG["Fade quest name that are not complete"],
					type = "toggle",
					width = "full",
					order = 2,
				},
				automaticAccept = {
					name = LQG["Accept quests automatically"],
					type = "select",
					values = module.automaticAcceptChoices,
					width = "full",
					order = 3,
				},
				automaticAcceptEscort = {
					name = LQG["Accept escort quest started by other player automatically"],
					type = "toggle",
					width = "full",
					order = 4,
				},
				automaticTurnIn = {
					name = LQG["Automatic quest turn-in"],
					type = "toggle",
					width = "full",
					order = 5,
				},
			},
		}
	end
	return options
end

function module:OnInitialize()
	self.db = core.db:RegisterNamespace("questsGivers", defaults, "Default")
	prdb = self.db.profile
	self:SetEnabledState(core:getModuleEnabled("questsGivers"))
	core:registerModuleOptions("questsGivers", getOptions, module.modName)
end

function module:OnEnable()
	for uid, v in pairs(LDQ) do
		self.dailies[v] = true
	end
	self:RegisterEvent("GOSSIP_SHOW")
	self:RegisterEvent("QUEST_GREETING")
	self:RegisterEvent("QUEST_COMPLETE")
	self:RegisterEvent("QUEST_DETAIL")
	self:RegisterEvent("QUEST_PROGRESS")
	self:RegisterEvent("QUEST_ACCEPT_CONFIRM")
end

function module:modifyQuestButton(button, title, level)
	if not button or not level or not title then return end
	if level == -1 then
		button:SetText("[*] "..title)
	else
		button:SetText("["..level.."] "..title)
	end
end

function module:fadeQuestButton(button, uid)
	if uid and not (core.quests[uid].complete[core.playerName] == 1 or core.quests[uid].numObjectives == 0) then
		button:SetVertexColor(0.5, 0.5, 0.5, 0.5)
	else
		button:SetVertexColor(1, 1, 1, 1)
	end
end

function module:clickQuest(button)
	if button:IsVisible() then
		local title = button:GetText()
		title = title:gsub("|c%x%x%x%x%x%x%x%x(.+)|r","%1")
		title = title:gsub("%[(.+)%] ", "")
		title = title:gsub("(.+) %((.+)%)", "%1")
		if button.type == "Available" and (prdb.automaticAccept == 1 or (prdb.automaticAccept == 2 and self.dailies[title]) or (prdb.automaticAccept == 3 and not self.dailies[title])) then
			button:Click()
		elseif button.type == "Active" and prdb.automaticTurnIn and ((core.questsTitleToUid[title] and (core.quests[core.questsTitleToUid[title]].complete[core.playerName] == 1 or core.quests[core.questsTitleToUid[title]].numObjectives == 0)) or (not core.questsTitleToUid[title])) then
			button:Click()
		end
	end
end



function module:loopAvailableQuests(...)
	local numQuests = select('#', ...)
	if numQuests == 0 then return end
	self.numQuests = self.numQuests + numQuests
	for i = 2, numQuests, 3 do
		self.buttonIndex = self.buttonIndex + 1
		local buttonName = "GossipTitleButton"..self.buttonIndex
		local button = _G[buttonName]
		local level = select(i, ...)
		local title = button:GetText()
		if prdb.addQuestLevel then
			self:modifyQuestButton(button, title, level)
		end
		_G[buttonName.."GossipIcon"]:SetVertexColor(1, 1, 1, 1)
	end
end

function module:loopActiveQuests(...)
	local numQuests = select('#', ...)
	if numQuests == 0 then return end
	self.numQuests = self.numQuests + numQuests
	for i=2, numQuests, 3 do
		self.buttonIndex = self.buttonIndex + 1
		local buttonName = "GossipTitleButton"..self.buttonIndex
		local button = _G[buttonName]
		local level = select(i, ...)
		local title = button:GetText()
		if prdb.addQuestLevel then
			self:modifyQuestButton(button, title, level)
		end
		if prdb.fadeNotCompletedQuests then
			self:fadeQuestButton(_G[buttonName.."GossipIcon"], core.questsTitleToUid[select(i-1, ...)])
		else
			_G[buttonName.."GossipIcon"]:SetVertexColor(1, 1, 1, 1)
		end
	end
end

function module:GOSSIP_SHOW()
	self.buttonIndex = 0
	self.numQuests = 0
	self:loopAvailableQuests(GetGossipAvailableQuests())
	self:loopActiveQuests(GetGossipActiveQuests())
	if IsShiftKeyDown() then return end
	for i = 1, self.numQuests do
		self:clickQuest(_G["GossipTitleButton" .. i])
	end
end

function module:QUEST_GREETING()
	local numActive, numAvailable = GetNumActiveQuests() or 0, GetNumAvailableQuests() or 0
	for i = 1, numActive do
		local title = GetActiveTitle(i)
		local level = GetActiveLevel(i)
		local buttonName = "QuestTitleButton"..i
		local button = _G[buttonName]
		if prdb.addQuestLevel then
			self:modifyQuestButton(button, title, level)
		end
		if prdb.fadeNotCompletedQuests then
			self:fadeQuestButton(_G[buttonName.."QuestIcon"], core.questsTitleToUid[title])
		else
			_G[buttonName.."QuestIcon"]:SetVertexColor(1, 1, 1, 1)
		end
	end
	for i = 1, numAvailable do
		local title = GetAvailableTitle(i)
		local level = GetAvailableLevel(i)
		local index = i + numActive
		local buttonName = "QuestTitleButton"..index
		self:modifyQuestButton(_G[buttonName], title, level)
		_G[buttonName.."QuestIcon"]:SetVertexColor(1, 1, 1, 1)
	end
	if IsShiftKeyDown() then return end
	for i = 1, numActive + numAvailable do
		self:clickQuest(_G['QuestTitleButton' .. i])
	end
end

function module:QUEST_DETAIL()
	local title = GetTitleText()
	if not IsShiftKeyDown() and (prdb.automaticAccept == 1 or (prdb.automaticAccept == 2 and self.dailies[title]) or (prdb.automaticAccept == 3 and not self.dailies[title])) then
		AcceptQuest()
	end
end

function module:QUEST_COMPLETE(event)
	if not IsShiftKeyDown() and prdb.automaticTurnIn and GetNumQuestChoices() <= 1 then
		GetQuestReward(QuestFrameRewardPanel.itemChoice)
	end
end

function module:QUEST_PROGRESS()
	if not IsShiftKeyDown() and prdb.automaticTurnIn and IsQuestCompletable() then
		CompleteQuest()
	end
end

function module:QUEST_ACCEPT_CONFIRM()
	if not IsShiftKeyDown() and prdb.automaticAcceptEscort then
		ConfirmAcceptQuest()
	end
end